// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 - 2006 Simon Goodall

package WFUT;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.*;

public class GUI extends JFrame implements WFUTCallbackHandler {
  private JPanel wfLogo;
  private JButton updateButton;
  private JButton listButton;
  private JButton exitButton;
  private JList channelList;
  private JScrollPane sp_channelList;
  private JTextArea chdesc;
  private JScrollPane sp_chdesc;
  private JPanel chlogo;
  private JTextField chname;
  private JTextField churl;
  private JTextField chemail;
  private JScrollPane scrollpane_1;
  private JLabel about;
  private JLabel label_1;
  private JLabel label_2;
  private JLabel label_3;
  private Image img;
  private Image imgLogo = null;
  private JScrollPane sp;
  private JTextArea console;
  public ProgressMonitor pm = null;  
  private UpdateThread ut;

  static GUI instance;

  static public GUI getInstance() { return instance; }

  public GUI() {
    instance = this;

    GUILayout customLayout = new GUILayout();
    //getContentPane().setFont(new Font("Courier New", Font.PLAIN, 12));
    getContentPane().setLayout(customLayout);

    // Load the WF logo and render it into a panel object 
    URL logoURL = GUI.class.getResource("/wf_logo_uta.png");
    img = Toolkit.getDefaultToolkit().getImage(logoURL);
    // SCALE_SMOOTH is not implemented on gcj yet
    img = img.getScaledInstance(80, 80, Image.SCALE_DEFAULT);
    wfLogo = new JPanel() {
      // Enable transparency
      public boolean isOpaque() {
        return true;
      }

      protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.drawImage(img, 0, 0, this);
      }
    };
    
    getContentPane().add(wfLogo);

    updateButton = new JButton(Bundle.getString("Update_Button"));
    updateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        // Do nothing if an update is currently in progress.
        if (ut != null && !ut.done){
          //System.out.println("Update already in progress");
          return;
        }
        // Disable updates until current update has finished.
        updateButton.setEnabled(false);
        
        // Get selected values does not seem to work under gcj
        // So get a single value and put into an array.
        Object ob = channelList.getSelectedValue();
        // Check that a value has been selected.
        if (ob != null) {
          Object obj[] = new Object[1];
          obj[0] = ob;
          //Create new thread to handle downloads
          ut = new UpdateThread();
          ut.addCallbackHandler(instance);
          ut.setChannels(obj);
          ut.start();          
        }
      }
    });   
    getContentPane().add(updateButton);

    listButton = new JButton(Bundle.getString("Get_List_Button"));
    listButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        try {
          final java.util.List l = Client.getInstance().listChannels();
          channelList.setListData(l.toArray());
        } catch (Exception e) {
          System.err.println(e);  
        }
      }
    });
    getContentPane().add(listButton);

    exitButton = new JButton(Bundle.getString("Exit_Button"));
    exitButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        System.exit(0);
      }
    });
    getContentPane().add(exitButton);
    
    DefaultListModel listModel_channelList = new DefaultListModel();
    channelList = new JList(listModel_channelList);
    channelList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        final Channel c = (Channel)channelList.getSelectedValue();
        if (c == null) return;
        // Setup the channel information fields
        chname.setText(c.getName());
        chname.setCaretPosition(0);        
        churl.setText(c.getURL());
        churl.setCaretPosition(0);        
        chdesc.setText(c.getDescription());
        chdesc.setCaretPosition(0);        
        chemail.setText(c.getEmail());
        chemail.setCaretPosition(0);
        
        // Make sure a logo is specified
        if (c.getLogo() == null) {
          // Do nothing
        } else if (c.getLogo().length() == 0) {
          // Do nothing
        } else {
          try {
            final String server = Options.getInstance().getServerRoot();
            final URL logoURL = new URL(server + c.getLogo());
            imgLogo = Toolkit.getDefaultToolkit().getImage(logoURL);
            if (imgLogo != null) {
              // SCALE_SMOOTH is not implemented on gcj yet
              imgLogo = imgLogo.getScaledInstance(chlogo.getWidth(),chlogo.getHeight(), Image.SCALE_DEFAULT);
              chlogo.repaint();
            }
          } catch (MalformedURLException mue) {
            System.err.println(mue);
          }
        }
      } 
    });
  
    channelList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);    
    sp = new JScrollPane(console);
    sp_channelList = new JScrollPane(channelList);
    sp_channelList.setViewportView(channelList);
    getContentPane().add(sp_channelList);
    
    chdesc = new JTextArea("");
    chdesc.setLineWrap(true);
    chdesc.setWrapStyleWord(true);
    
    sp_chdesc = new JScrollPane(chdesc);
    getContentPane().add(sp_chdesc);
    
    chlogo = new JPanel() {
      public boolean isOpaque() {
        return true;
      }

      protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        try {
          if (imgLogo != null && this.isVisible())
            g.drawImage(imgLogo, 0, 0, this);
        } catch (Exception e) {
          System.out.println(e);
        }
      }
    };
    getContentPane().add(chlogo);

    chname = new JTextField("");
    getContentPane().add(chname);

    churl = new JTextField("");
    getContentPane().add(churl);

    chemail = new JTextField("");
    getContentPane().add(chemail);

    scrollpane_1 = new JScrollPane();
    getContentPane().add(scrollpane_1);

    about = new JLabel("By Simon Goodall");
    getContentPane().add(about);

    label_1 = new JLabel(Bundle.getString("Name_label"));
    getContentPane().add(label_1);

    label_2 = new JLabel(Bundle.getString("WebPage_label"));
    getContentPane().add(label_2);

    label_3 = new JLabel(Bundle.getString("E_Mail_label"));
    getContentPane().add(label_3);

    console = new JTextArea() {
    // Override normal append method to allow threads to call this method.
	public void append(String text) {
        append2(text);
      }
      public synchronized void append2(String text) {
        super.append(text);
        this.setCaretPosition(this.getCaretPosition()+text.length());
      }
    };

    sp = new JScrollPane(console);
    sp.setViewportView(console);
    sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    getContentPane().add(sp);

    setSize(getPreferredSize());

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });

    setTitle(Bundle.getString("Main_title"));
  }
  
  // Callback handler functions
  public void DownloadingFile(final String filename) {
    console.append("Downloading: " + filename + "\n");
    
    pm.setProgress(ut.filenum);
    if (ut.current != null) {
      int i = ut.current.getFilename().lastIndexOf("/");
      if (i > 0) ++i;
      else i = 0;
      final String txt = ut.current.getFilename().substring(i);
      pm.setNote(txt);
    }
  }

  public void DownloadFailed(final String filename) {
    console.append("Error Downloading: " + filename + "\n");
  }  
  
  public void DownloadComplete(final String filename) {
    console.append("Download Complete: " + filename + "\n");    
  }
  
  public void ChannelBegin(final String channel, int numFiles) {
    console.append("Downloading Channel: " + channel + "\n");
    pm = new ProgressMonitor(this, "Updating " + channel, "", 0, numFiles);
    pm.setProgress(0);
    pm.setMillisToDecideToPopup(1);
  }
  
  public void ChannelCompleted(final String channel) {
    console.append("Channel Complete: " + channel + "\n");
    pm.close();
    ut.running = false;
    pm = null;
    //Re-enable update button
    updateButton.setEnabled(true);
  }
  
  public void Error(final String msg) {
    console.append("Error: " + msg + "\n");
  }
}

class GUILayout implements LayoutManager {

    public GUILayout() {
    }

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        Insets insets = parent.getInsets();
        dim.width = 512 + insets.left + insets.right;
        dim.height = 339 + insets.top + insets.bottom;

        return dim;
    }

    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);
        return dim;
    }

    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();

        Component c;
        c = parent.getComponent(0);
        if (c.isVisible()) {c.setBounds(insets.left+8,insets.top+8,96,88);}
        c = parent.getComponent(1);
        if (c.isVisible()) {c.setBounds(insets.left+8,insets.top+136,96,24);}
        c = parent.getComponent(2);
        if (c.isVisible()) {c.setBounds(insets.left+8,insets.top+104,96,24);}
        c = parent.getComponent(3);
        if (c.isVisible()) {c.setBounds(insets.left+8,insets.top+208,96,24);}
        c = parent.getComponent(4);
        if (c.isVisible()) {c.setBounds(insets.left+112,insets.top+8,136,192);}
        c = parent.getComponent(5);
        if (c.isVisible()) {c.setBounds(insets.left+256,insets.top+128,248,72);}
        c = parent.getComponent(6);
        if (c.isVisible()) {c.setBounds(insets.left+344,insets.top+0,72,24);}
        c = parent.getComponent(7);
        if (c.isVisible()) {c.setBounds(insets.left+328,insets.top+32,168,24);}
        c = parent.getComponent(8);
        if (c.isVisible()) {c.setBounds(insets.left+328,insets.top+64,168,24);}
        c = parent.getComponent(9);
        if (c.isVisible()) {c.setBounds(insets.left+328,insets.top+96,168,24);}
        c = parent.getComponent(10);
        if (c.isVisible()) {c.setBounds(insets.left+256,insets.top+128,248,72);}
        c = parent.getComponent(11);
        if (c.isVisible()) {c.setBounds(insets.left+224,insets.top+208,280,24);}
        c = parent.getComponent(12);
        if (c.isVisible()) {c.setBounds(insets.left+256,insets.top+32,72,24);}
        c = parent.getComponent(13);
        if (c.isVisible()) {c.setBounds(insets.left+256,insets.top+64,72,24);}
        c = parent.getComponent(14);
        if (c.isVisible()) {c.setBounds(insets.left+256,insets.top+96,72,24);}
        c = parent.getComponent(15);
        if (c.isVisible()) {c.setBounds(insets.left+8,insets.top+235,490,100);}
    }
      
}
